#include "Expression.h"

std::string TrueExpression::toString() const {
    return "true";
}

Type TrueExpression::accept(ExpressionVisitor<Type, TypeError>& v) {
    return v.visit(*this);
}

std::string FalseExpression::toString() const {
    return "false";
}

Type FalseExpression::accept(ExpressionVisitor<Type, TypeError>& v) {
    return v.visit(*this);
}

IntExpression::IntExpression(int val) : value(val) {
    assert(val >= 0);
}

std::string IntExpression::toString() const {
    return std::to_string(value);
}

Type IntExpression::accept(ExpressionVisitor<Type, TypeError>& v) {
    return v.visit(*this);
}

NegExpression::NegExpression(Expression* expression) : e(expression) {
}

std::string NegExpression::toString() const {
    std::stringstream ss;
    ss << "- (" << e->toString() << ")";
    return ss.str();
}

Type NegExpression::accept(ExpressionVisitor<Type, TypeError>& v) {
    return v.visit(*this);
}

AddExpression::AddExpression(Expression* expr1, Expression* expr2) : e1(expr1), e2(expr2) {
}

std::string AddExpression::toString() const {
    std::stringstream ss;
    ss << "(" << e1->toString() << ") + (" << e2->toString() << ")";
    return ss.str();
}

Type AddExpression::accept(ExpressionVisitor<Type, TypeError>& v) {
    return v.visit(*this);
}

OrExpression::OrExpression(Expression* expr1, Expression* expr2) : e1(expr1), e2(expr2) {
}

std::string OrExpression::toString() const {
    std::stringstream ss;
    ss << "(" << e1->toString() << ") || (" << e2->toString() << ")";
    return ss.str();
}

Type OrExpression::accept(ExpressionVisitor<Type, TypeError>& v) {
    return v.visit(*this);
}

EqualsExpression::EqualsExpression(Expression* expr1, Expression* expr2) : e1(expr1), e2(expr2) {
}

std::string EqualsExpression::toString() const {
    std::stringstream ss;
    ss << "(" << e1->toString() << ") == (" << e2->toString() << ")";
    return ss.str();
}

Type EqualsExpression::accept(ExpressionVisitor<Type, TypeError>& v) {
    return v.visit(*this);
}

IfThenElseExpression::IfThenElseExpression(Expression* condition, Expression* thenExpr, Expression* elseExpr)
    : cond(condition), then(thenExpr), else_(elseExpr) {
}

std::string IfThenElseExpression::toString() const {
    std::stringstream ss;
    ss << "if (" << cond->toString() << ") then (" << then->toString() << ") else (" << else_->toString() << ")";
    return ss.str();
}

Type IfThenElseExpression::accept(ExpressionVisitor<Type, TypeError>& v) {
    return v.visit(*this);
}
